/*
 * LC7461M8100 simulator
 *
 * A. Hojka
 * November 2018
 * */


#include "stm32l0xx.h"
#include "sanyo_ir.h"


/* --------------------------------------------------------
 * 		(MODIFIED NEC- 'extended NEC' - Sanyo) PROTOCOL DETAILS
 * --------------------------------------------------------
 *
 * T - 38kHz IR pulse time  = 0,5625ms
 *
 * ->transmission of logical 0 takes 2 * T = 1,125ms
 *    ___     _
 *  _/   \___/
 *     T   T			= 2 * T
 *
 * ->transmission of logical 1 takes 4 * T = 2,25ms
 *   ___           _
 * _/   \_________/    = 4 * T
 *    T   T  T  T
 *
 * -------------------------------------------
 * Normal frame (command frame)
 *--------------------------------------------
 * Leading burst (Start-Of-Frame)			= 9ms		= 16 * T
 * Space									= 4,5ms		= 8 * T
 * 13 bits of CustomCode and ~(CustomCode) 	= 43,875ms 	= 13 * 6 * T = 78 * T
 * 8 bits of KeyData and ~(KeyData) 		= 27ms  	= 8 * 6 * T = 48 * T
 * Final pulse (End-Of-Frame)				= 0,5625ms 	= 1 * T
 *										-----------------------------
 *		sum of frame items					= 84,9375ms = 151 * T
 * 		inter-frame space					= 23,0625ms	= 41 * T
 * 										-----------------------------
 * 				TOTAL						= 108ms		= 192 * T
 *
 * --------------------------------------------
 * When key-down continued (repeated frame)
 * --------------------------------------------
 * Leading burst (Start-Of-Frame)			= 9ms		= 16 * T
 * Space									= 4,5ms		= 8 * T
 * Final pulse (End-Of-Frame)				= 0,5625ms 	= 1 * T
 *										-----------------------------
 *		sum of frame items					= 14,0625	= 25 * T
 * 		inter-frame space					= 93,9375	= 167 * T
 * 										-----------------------------
 * 				TOTAL						= 108ms		= 192 * T
 *
 * buffer for IR frame 	--> 2 + (2 x 26) + (2 x 16) + 1 + 1 + 1(EOF) = 89B
 *
 */

static void IREncodeBits(uint8_t *bufptr, uint16_t usData, uint8_t ucDlen)
{
uint8_t ucidx = 0;

	do{
		*bufptr++ = 1;					//IR pulse at the beginning of bit

		if( usData & (1 << ucidx) ){
			*bufptr++ = 3;				//wait 3 T
		}else{
			*bufptr++ = 1;				//wait T
		}

	}while(++ucidx != ucDlen);
}

/*this can be done only once - fixed part of frame*/
void IREncodeCCode(uint8_t *bufptr, uint16_t usCustomcode)
{
	bufptr[0] = 16;									//9ms IR burst
	bufptr[1] = 8;									//4.5ms space
	IREncodeBits(bufptr + 2, usCustomcode, 13);
	IREncodeBits(bufptr + 2 + (2*13), ~usCustomcode, 13);
}

/*data varies, encoded before each transmission*/
void IREncodeKData(uint8_t *bufptr, uint8_t ucKeyData)
{
	IREncodeBits(bufptr, ucKeyData, 8);
	IREncodeBits(bufptr + (2*8), ~ucKeyData, 8);

	bufptr[32] = 1;		//stop
	bufptr[33] = 41;	//dead time
	bufptr[34] = 0;		//EOF
}

/*make entire frame - first frame with data*/
void IREncodeFrame(uint8_t *bufptr, uint16_t usCustomcode, uint8_t ucKeyData)
{
	IREncodeCCode(bufptr, usCustomcode);	//2*26+2 = 54
	IREncodeKData(bufptr + 54, ucKeyData);	//2*16 + 3 = 35	--> 89B
}

/*make repeated frame - "hand made" */
void IREncodeRepFrame(uint8_t *bufptr)
{
	*bufptr++ = 16;		//9ms IR burst
	*bufptr++ = 8;		//4.5ms space
	*bufptr++ = 1;		//single T IR pulse
	*bufptr++ = 167;	//dead time
	*bufptr = 	0;		//EOF
}



